<?php

namespace Zen\Payment;

/**
 * Class Api
 *
 * @package Zen\Payment
 */
class Api
{

	const TRANSACTIONS = 'transactions';
	const REFUND = 'refund';

	/**
	 * @var array
	 */
	private static $serviceUrls = [
		Util::ENVIRONMENT_PRODUCTION => 'https://api.zen.com/v1',
		Util::ENVIRONMENT_SANDBOX    => 'https://api.zen-test.com/v1',
	];

	/**
	 * @var string
	 */
	private $authorizationToken;

	/**
	 * Api constructor.
	 *
	 * @param string $authorizationToken
	 */
	public function __construct($authorizationToken)
	{

		$this->authorizationToken = $authorizationToken;
	}

	/**
	 * @param string $environment
	 * @param string $amount
	 * @param string $transactionUuid
	 * @param string $currency
	 * @param string $orderId
	 *
	 * @return array|bool|string
	 */
	public function createRefund($environment, $amount, $transactionUuid, $currency, $orderId)
	{

		return $this->call(
			$this->getRefundCreateUrl($environment),
			Util::METHOD_REQUEST_POST,
			$this->prepareRefundData($amount, $transactionUuid, $currency, $orderId),
			true
		);
	}

	/**
	 * @param string $url
	 * @param string $methodRequest
	 * @param string $body
	 * @param bool   $decode
	 *
	 * @return bool|string|array
	 */
	private function call($url, $methodRequest, $body, $decode = false)
	{

		$curl = curl_init($url);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $methodRequest);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
		curl_setopt($curl, CURLOPT_TIMEOUT, 10);
		curl_setopt($curl, CURLOPT_HTTPHEADER, [
			'Content-Type: application/json',
			'Authorization: ' . $this->authorizationToken,
		]);

		$resultCurl = curl_exec($curl);

		$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

		if(($httpCode >= 300) || !$resultCurl) {
			return [
				'success' => false,
				'data'    => [
					'httpCode' => $httpCode,
					'error'    => curl_error($curl),
				],
			];
		}

		if($decode) {
			return [
				'success' => true,
				'body'    => json_decode($resultCurl, true),
			];
		}

		return [
			'success' => true,
			'body'    => $resultCurl,
		];
	}

	/**
	 * @param string $environment
	 *
	 * @return string
	 */
	private function getRefundCreateUrl($environment)
	{

		$baseUrl = self::getServiceUrl($environment);

		if($baseUrl) {
			return $baseUrl . '/' . self::TRANSACTIONS . '/' . self::REFUND;
		}

		return '';
	}

	/**
	 * @param string $environment
	 *
	 * @return string
	 */
	public static function getServiceUrl($environment)
	{

		if(isset(self::$serviceUrls[$environment])) {
			return self::$serviceUrls[$environment];
		}

		return '';
	}

	/**
	 * @param mixed  $amount
	 * @param string $transactionUuid
	 * @param string $currency
	 * @param string $orderId
	 *
	 * @return string
	 */
	private function prepareRefundData(
		$amount,
		$transactionUuid,
		$currency,
		$orderId
	) {

		return json_encode([
			'amount'                => (string) $amount,
			'transactionId'         => $transactionUuid,
			'currency'              => $currency,
			'merchantTransactionId' => $orderId . '#' . uniqid(),
		]);
	}
}
